<?php

class Registration {

    private $db;
    private $session;

    public function __construct($db, $session, $configs, $functions, $logger) {
        $this->db = $db;
        $this->session = $session;
        $this->configs = $configs;
        $this->functions = $functions;
        $this->logger = $logger;
    }

    /* ****************************************************************************************************************************
     * register - Gets called when the user has just submitted the registration form. Determines if there were any errors with the 
     * entry fields, if so, it records the errors and returns to the form. If no errors were found, it registers the new user and 
     * returns a success code depending on what type of activation it is. It returns 2 if registration failed.
     * ****************************************************************************************************************************
     */
    function register($user, $firstname, $lastname, $pass, $conf_pass, $email, $isadmin, $userlvl) {
        /* Username error checking */
        $field = "user";  //Use field name for username
        if (!$user || strlen($user = trim($user)) == 0) {
            Form::setError($field, "Username not entered");
        } else {
            /* check length */
            if (strlen($user) < $this->configs->getConfig('min_user_chars')) {
                Form::setError($field, "Username below " . $this->configs->getConfig('min_user_chars') . " characters");
            } else if (strlen($user) > $this->configs->getConfig('max_user_chars')) {
                Form::setError($field, "Username above " . $this->configs->getConfig('max_user_chars') . " characters");
            }
            /* Check username contains correct characters (Regex is set in configs) */ 
            else if (!$this->functions->usernameRegex($user)) {
                Form::setError($field, "Username does not match requirements");
            }
            /* Check if username is reserved */ 
            else if (strcasecmp($user, GUEST_NAME) == 0) {
                Form::setError($field, "Username reserved word");
            }
            /* Check if username is already in use */ 
            else if ($this->usernameTaken($user, $this->db)) {
                Form::setError($field, "Username already in use");
            }
            /* Check if username is disallowed */ 
            else if ($this->usernameDisallowed($user)) {
                Form::setError($field, "Username Disallowed");
            } 
        }

        /* Firstname error checking */
        $this->functions->nameCheckNotRequired($firstname, 'firstname', 'First Name', 2, 30);

        /* Lastname error checking */
        $this->functions->nameCheckNotRequired($lastname, 'lastname', 'Last Name', 2, 30);
        
        /* Email error checking */
        $this->functions->emailCheck($email, 'email');

        /* Password error checking */
        $field = "pass";  //Use field name for password
        if (!$pass) {
            Form::setError($field, "Password not entered");
        } else {
            /* Check length */
            if (strlen($pass) < $this->configs->getConfig('min_pass_chars')) {
                Form::setError($field, "Password too short");
            }
            /* Check if password is too long */ 
            else if (strlen($pass) > $this->configs->getConfig('max_pass_chars')) {
                Form::setError($field, "Password too long");
            }
            /* Check if passwords match */ 
            else if ($pass != $conf_pass) {
                Form::setError($field, "Passwords do not match");
            }
        }    

        /* Errors exist, have user correct them */
        if (Form::$num_errors > 0) {
            $_SESSION['value_array'] = $_POST;
            $_SESSION['error_array'] = Form::getErrorArray();
            return 1;  // Errors with form
        }
        /* No errors, add the new account to the database */ 
        else {          
            if ($this->addNewUser($user, $firstname, $lastname, $pass, $email)) {
                
                // $mailer = new Mailergmail($this->db, $this->configs);
                $time = time();

                /* Check Account activation setting and process accordingly. */ 

                /* No Activation Needed but E-mail going out */ 
                $successcode = 0; 
                
                /* Update Log */
                if($isadmin == '1'){
                    $id = $this->session->id;
                    $this->logger->logAction($id, "REGISTERED : ".$user );
                } else {
                    $id = $this->functions->getUserInfoSingular('id', $user);
                    $this->logger->logAction($id, "REGISTERED" );
                }
                
                return $successcode;  //New user added succesfully
            } else {
                return 2;  //Registration attempt failed
            }
        }
    }

    /* *********************************************************************************************
     * usernameTaken - Returns true if the username has been taken by another user, false otherwise.
     * *********************************************************************************************
     */
    function usernameTaken($username) {
        $result = $this->db->query("SELECT username FROM users WHERE username = '$username'");
        $count = $result->rowCount();
        return ($count > 0);
    }

    /* ***********************************************************************
     * usernameDisallowed - Returns true if the username has been disallowed.
     * ***********************************************************************
     */
    function usernameDisallowed($username) {
        $query = "select * from banlist where :username like concat('%',ban_username,'%') AND ban_username != ''";
        $stmt = $this->db->prepare($query);
        $stmt->bindValue(':username', $username);
        $stmt->execute();
        $count = $stmt->rowCount();
        return ($count > 0);
    }

    /* ***********************************************************************************
     * addNewUser - Inserts the given (username, password, email) info into the database. 
     * Appropriate user level is set. Returns true on success, false otherwise.
     * ***********************************************************************************
     */
    function addNewUser($username, $firstname, $lastname, $password, $email) {
        $time = time(); 
        $ulevel = 1; 
        /* Hash password using PHP's inbuilt password_hash function - currently using BCRYPT - as of 2.5 */ 
        $password_hash = password_hash($password, PASSWORD_DEFAULT);
        
        // $userip = $_SERVER['REMOTE_ADDR'];

        $query = "INSERT INTO users SET username = :username, firstname = :firstname, lastname = :lastname, password = :password, userlevel = $ulevel, email = :email, timestamp = $time, regdate = $time";
        $stmt = $this->db->prepare($query);
        return $stmt->execute(array(':username' => $username, ':firstname' => $firstname, ':lastname' => $lastname, ':password' => $password_hash, ':email' => $email));
    }

}
